@@ -634,13 +634,13 @@ void on_about(GtkMenuItem *menuitem,
                                  NULL };
     gtk_show_about_dialog (NULL,
 			   "program-name", "Gromit-MPX",
-			   "logo-icon-name", "gromit-mpx",
+			   "logo-icon-name", "net.christianbeier.Gromit-MPX",
 			   "title", "About Gromit-MPX",
 			   "comments", "Gromit-MPX (GRaphics Over MIscellaneous Things - Multi-Pointer-EXtension) is an on-screen annotation tool that works with any Unix desktop environment under X11 as well as Wayland.",
 			   "version", VERSION,
 			   "website", "https://github.com/bk138/gromit-mpx",
 			   "authors", authors,
-			   "copyright", "2009-2020 Christian Beier, Copyright 2000 Simon Budig",
+			   "copyright", "2009-2022 Christian Beier, Copyright 2000 Simon Budig",
 			   "license-type", GTK_LICENSE_GPL_2_0,
 			   NULL);
 }
