@@ -307,6 +307,12 @@ gint reshape (gpointer user_data)
 	  gtk_widget_input_shape_combine_region(data->win, r);
 	  cairo_region_destroy(r);
 
+	  /*
+	    this is not needed when there is user input, i.e. when drawing,
+	    but needed when uing the undo functionality.
+	  */
+	  gtk_widget_queue_draw(data->win);
+
           data->modified = 0;
           data->delayed = 0;
         }
@@ -528,6 +534,8 @@ void undo_drawing (GromitData *data)
   GdkRectangle rect = {0, 0, data->width, data->height};
   gdk_window_invalidate_rect(gtk_widget_get_window(data->win), &rect, 0); 
 
+  data->modified = 1;
+
   if(data->debug)
     g_printerr ("DEBUG: Undo drawing %d.\n", data->undo_head);
 }
@@ -550,6 +558,8 @@ void redo_drawing (GromitData *data)
   GdkRectangle rect = {0, 0, data->width, data->height};
   gdk_window_invalidate_rect(gtk_widget_get_window(data->win), &rect, 0); 
 
+  data->modified = 1;
+
   if(data->debug)
     g_printerr("DEBUG: Redo drawing.\n");
 }
@@ -689,8 +699,9 @@ void main_do_event (GdkEventAny *event,
 
 
 
-void setup_main_app (GromitData *data, gboolean activate)
+void setup_main_app (GromitData *data, int argc, char ** argv)
 {
+  gboolean activate;
 
   if(getenv("GDK_CORE_DEVICE_EVENTS")) {
       g_printerr("GDK is set to not use the XInput extension, Gromit-MPX can not work this way.\n"
@@ -698,6 +709,27 @@ void setup_main_app (GromitData *data, gboolean activate)
       exit(1);
   }
 
+  /*
+    HOT KEYS
+  */
+  data->hot_keyval = DEFAULT_HOTKEY;
+  data->hot_keycode = 0;
+
+  data->undo_keyval = DEFAULT_UNDOKEY;
+  data->undo_keycode = 0;
+
+  char *xdg_current_desktop = getenv("XDG_CURRENT_DESKTOP");
+  if (xdg_current_desktop && strcmp(xdg_current_desktop, "XFCE") == 0) {
+      /*
+	XFCE per default grabs Ctrl-F1 to Ctrl-F12 (switch to workspace 1-12)
+	and Alt-F9 (minimize window) which renders Gromit-MPX's default hotkey
+	mapping unusable. Provide different defaults for that desktop environment.
+      */
+      data->hot_keyval = DEFAULT_HOTKEY_XFCE;
+      data->undo_keyval = DEFAULT_UNDOKEY_XFCE;
+      g_print("Detected XFCE, changing default hot keys to '" DEFAULT_HOTKEY_XFCE "' and '" DEFAULT_UNDOKEY_XFCE "'\n");
+  }
+
   /* COLOURS */
   g_free(data->white);
   g_free(data->black);
@@ -817,14 +849,18 @@ void setup_main_app (GromitData *data, gboolean activate)
    */
   data->tool_config = g_hash_table_new (g_str_hash, g_str_equal);
   parse_config (data);
-  if (data->debug)
-    g_hash_table_foreach (data->tool_config, parse_print_help, NULL);
+  g_hash_table_foreach (data->tool_config, parse_print_help, NULL);
 
   /*
     parse key file
   */
   read_keyfile(data);
 
+  /*
+    parse cmdline
+  */
+  activate = app_parse_args (argc, argv, data);
+
   // might have been in key file
   gtk_widget_set_opacity(data->win, data->opacity);
 
@@ -919,7 +955,7 @@ void setup_main_app (GromitData *data, gboolean activate)
      TRAY ICON
   */
   data->trayicon = app_indicator_new (PACKAGE_NAME,
-				      PACKAGE_NAME,
+				      "net.christianbeier.Gromit-MPX",
 				      APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
 
   app_indicator_set_status (data->trayicon, APP_INDICATOR_STATUS_ACTIVE);
@@ -1086,24 +1122,6 @@ int app_parse_args (int argc, char **argv, GromitData *data)
    gboolean  wrong_arg = FALSE;
    gboolean  activate = FALSE;
 
-   data->hot_keyval = DEFAULT_HOTKEY;
-   data->hot_keycode = 0;
-
-   data->undo_keyval = DEFAULT_UNDOKEY;
-   data->undo_keycode = 0;
-
-   char *xdg_current_desktop = getenv("XDG_CURRENT_DESKTOP");
-   if (xdg_current_desktop && strcmp(xdg_current_desktop, "XFCE") == 0) {
-       /*
-	 XFCE per default grabs Ctrl-F1 to Ctrl-F12 (switch to workspace 1-12)
-	 and Alt-F9 (minimize window) which renders Gromit-MPX's default hotkey
-	 mapping unusable. Provide different defaults for that desktop environment.
-       */
-       data->hot_keyval = DEFAULT_HOTKEY_XFCE;
-       data->undo_keyval = DEFAULT_UNDOKEY_XFCE;
-       g_print("Detected XFCE, changing default hot keys to '" DEFAULT_HOTKEY_XFCE "' and '" DEFAULT_UNDOKEY_XFCE "'\n");
-   }
-
    for (i=1; i < argc ; i++)
      {
        arg = argv[i];
@@ -1374,7 +1392,7 @@ int main (int argc, char **argv)
     return main_client (argc, argv, data);
 
   /* Main application */
-  setup_main_app (data, app_parse_args (argc, argv, data));
+  setup_main_app (data, argc, argv);
   gtk_main ();
   shutdown_input_devices(data);
   write_keyfile(data); // save keyfile config
@@ -1385,7 +1403,7 @@ int main (int argc, char **argv)
 void indicate_active(GromitData *data, gboolean YESNO)
 {
     if(YESNO)
-	app_indicator_set_icon(data->trayicon, "gromit-mpx_active");
+	app_indicator_set_icon(data->trayicon, "net.christianbeier.Gromit-MPX.active");
     else
-	app_indicator_set_icon(data->trayicon, PACKAGE_NAME);
+	app_indicator_set_icon(data->trayicon, "net.christianbeier.Gromit-MPX");
 }
